import sys

import hashlib
import os
import platform
import requests
import zipfile


class ProgressBar(object):

    def __init__(self, title,
                 count=0.0,
                 run_status=None,
                 fin_status=None,
                 total=100.0,
                 unit='', sep='/',
                 chunk_size=1.0):
        super(ProgressBar, self).__init__()
        self.info = "%s %s %.2f %s %s %.2f %s"
        self.title = title
        self.total = total
        self.count = count
        self.chunk_size = chunk_size
        self.status = run_status or ""
        self.fin_status = fin_status or " " * len(self.status)
        self.unit = unit
        self.seq = sep

    def __get_info(self):
        _info = self.info % (self.title, self.status,
                             self.count / self.chunk_size, self.unit, self.seq, self.total / self.chunk_size, self.unit)
        return _info

    def refresh(self, count=1, status=None):
        self.count += count
        self.status = status or self.status
        end_str = "\r"
        if self.count >= self.total:
            end_str = '\n'
            self.status = status or self.fin_status
        # sys.stdout.write(self.__get_info())
        # sys.stdout.flush()
        print(self.__get_info(), end=end_str)


def download_batch(data, name="", url_pre=""):
    '''
    调用该方法下载CSV中的数据
    :参数 data: 要下载的数据
    :参数 name:下载字段的名字
    :参数 url_pre:连接添加前缀
    :下载时，会打印下载进度
    '''
    total = len(data)
    fail = 0
    local = 0
    if name is not "":
        print("column [" + name + "] starting download")
    for index in range(len(data)):
        try:
            res = check_download(url_pre + data[index])
            local += res
            progress(total, index + 1)
        except Exception as e:
            print(e)
            fail += 1
    print("\nLocal count:" + str(local) + " Failed count:" + str(fail))


def genearteMD5(str):
    hl = hashlib.md5()
    hl.update(str.encode(encoding='utf-8'))
    return hl.hexdigest()


def check_download(url, data_dir, cache=True):
    sys = platform.system()
    if sys == "Windows":
        cache_path = os.path.join(os.environ['HOMEPATH'], '.flyai', 'cache', genearteMD5(url))
    else:
        cache_path = os.path.join(os.environ['HOME'], '.flyai', 'cache', genearteMD5(url))

    if not os.path.exists(cache_path):
        os.makedirs(cache_path)
    cache_path = os.path.join(cache_path, os.path.basename(url))
    if not os.path.exists(data_dir):
        os.makedirs(data_dir)
    path = os.path.join(data_dir, os.path.basename(url))

    if 'http' in url:
        r = requests.get(url, stream=True)
        chunk_size = 1024  # 单次请求最大值
        content_size = int(r.headers['content-length'])  # 内容体总大小
        if os.path.exists(cache_path):
            file_size = os.path.getsize(cache_path)
        else:
            file_size = 0
        if content_size == file_size and cache:
            print(os.path.basename(url), "use cached")
        else:
            print(os.path.basename(url), "total size: ", content_size // 1024 // 1024, 'MB')
            progress = ProgressBar(os.path.basename(url), total=content_size,
                                   unit="MB", chunk_size=chunk_size * 1024, run_status="downloading",
                                   fin_status="Download completed")
            with open(cache_path, "wb") as data:
                for chunk in r.iter_content(chunk_size=1024):
                    if chunk:
                        data.write(chunk)
                        progress.refresh(count=len(chunk))
        if 'zip' in os.path.basename(url):
            un_zip(cache_path, data_dir)

    if 'zip' in os.path.basename(url):
        return os.path.splitext(path)[0] + ".csv"
    elif 'http' not in url:
        return os.path.join(data_dir, url)
    else:
        return path


def un_zip(file_name, data_dir):
    zip_file = zipfile.ZipFile(file_name)
    if not os.path.exists(file_name):
        os.makedirs(file_name)
    for names in zip_file.namelist():
        zip_file.extract(names, data_dir)
    zip_file.close()


def progress(total, current):
    pr = int((100 / total) * current)
    now = "\r[%s%s]%d%%" % (
        "*" * pr, " " * (100 - pr), pr)
    sys.stdout.write(now)
    sys.stdout.flush()
