# -*- coding: utf-8 -*

import pandas as pd

from flyai.processor.download import check_download
from flyai.source.base import Base, DATA_PATH
from flyai.utils.yaml_helper import Yaml


class Csv(Base):
    def __init__(self, config):
        self.page = 1
        train_path = check_download(config['train_url'], DATA_PATH)
        self.data = pd.read_csv(train_path)
        print("load train csv:", train_path)
        test_path = check_download(config['test_url'], DATA_PATH)
        self.test = pd.read_csv(test_path)
        print("load test csv:", test_path)
        self.x_test = None
        self.y_test = None

    def next_batch(self, size=32):
        x_train, y_train = self.get_train_data(size)
        x_test, y_test = self.get_validation_data()
        return x_train, y_train, x_test, y_test

    def get_train_data(self, size=32):
        train = self.data[(int(self.page) - 1) * int(size): (int(self.page) * int(size))]
        self.page += 1
        if len(train) < size:
            self.page = 1
            train = self.data[(int(self.page) - 1) * int(size): (int(self.page) * int(size))]
        x_train, y_train = self.get_data(train)
        return x_train, y_train

    def get_all_data(self):
        x_train, y_train = self.get_data(self.data)
        x_test, y_test = self.get_data(self.test)
        return x_train, y_train, x_test, y_test

    def get_validation_data(self):
        if self.x_test is None or self.y_test is None:
            validation = self.test
            self.x_test, self.y_test = self.get_data(validation)
        return self.x_test, self.y_test

    def get_evaluate_data(self, path):
        evaluate_path = check_download(path, DATA_PATH)
        evaluate = pd.read_csv(evaluate_path)
        return self.get_data(evaluate)

    def get_data(self, data):
        yaml = Yaml()
        x_names = yaml.get_input_names()
        y_names = yaml.get_output_names()
        x_data = data[x_names]
        y_data = data[y_names]
        x_data = x_data.to_dict(orient='records')
        y_data = y_data.to_dict(orient='records')
        return x_data, y_data
