# -*- coding: utf-8 -*
import sys
from time import time

import json
import os
import random
import requests
from flyai.source.base import Base, DATA_PATH
from flyai.utils.yaml_helper import Yaml


class Source(Base):
    def __init__(self, custom_source=None):
        yaml = Yaml()
        try:
            f = open(os.path.join(sys.path[0], 'train.json'))
            line = f.readline().strip()
        except IOError:
            line = ""

        postdata = {'id': yaml.get_data_id(),
                    'env': line,
                    'time': time(),
                    'sign': random.random()}
        servers = yaml.get_servers()
        r = requests.post(servers[0]['url'] + "/dataset", data=postdata)
        self.source = json.loads(r.text)
        if 'yaml' in self.source:
            self.source = self.source['yaml']
            if custom_source is None:
                self.source = self.create_instance("flyai.source." + self.source['type'] + "_source",
                                                   self.source['type'].capitalize(), self.source['config'])
            else:
                self.source = custom_source
        else:
            if not os.path.exists(os.path.join(DATA_PATH, "train.csv")) and not os.path.exists(
                    os.path.join(DATA_PATH, "test.csv")):
                raise Exception("invalid data id!")
            else:
                self.source = self.create_instance("flyai.source.csv_source", 'Csv',
                                                   {'train_url': os.path.join(DATA_PATH, "train.csv"),
                                                    'test_url': os.path.join(DATA_PATH, "test.csv")})

    def next_batch(self, size=32):
        return self.source.next_batch(size)

    def get_train_data(self, size=32):
        return self.source.get_train_data(size)

    def get_validation_data(self):
        return self.source.get_validation_data()

    def get_evaluate_data(self, path):
        return self.source.get_evaluate_data(path)

    def get_all_data(self):
        return self.source.get_all_data()

    def create_instance(self, module_name, class_name, *args, **kwargs):
        module_meta = __import__(module_name, globals(), locals(), [class_name])
        class_meta = getattr(module_meta, class_name)
        return class_meta(*args, **kwargs)

    def get_method(self, clz, method_name, *args):
        m = getattr(clz, method_name)
        return m(*args)
